#!/usr/bin/perl

use Data::Dumper;

my $TERMS = 50; # size of the pool (top words by infogain)
my $TAGS  = 50; # final number of tags (top words by occurrence)
my $SIZENUM = 6; # number of size classes to be used in the HTML document,
		# represented as different font sizes in the CSS

my $FIREFOX_BIN		= '/usr/bin/firefox';
my $RAINBOW_BIN		= '/usr/bin/rainbow';
my $DIR_MODEL		= './results/model';
my $DIR_DATA		= './train';
my $DIR_TEST		= $ARGV[0];
my $FILE_STOPLIST	= './stopwords.txt';
my $FILE_TMPDATA	= './results/data.txt';
my $HTML_TEMPLATE	= './template.html';
my $HTML_OUTPUT		= './results/output.html';

die "Usage: rain.pl <test directory>\n\n" unless $ARGV[0];

print "-----------------------------------------------------------------------------\n";
print "rain.pl v0.99\n\n";
print "[i] Getting input data from directory $DIR_TEST\n";

`rm -rf $DIR_MODEL`;

`$RAINBOW_BIN -d $DIR_MODEL --append-stoplist-file $FILE_STOPLIST -h --index $DIR_DATA $DIR_TEST`;

`$RAINBOW_BIN -d $DIR_MODEL -T $TERMS --print-matrix=siw >$FILE_TMPDATA`;

print "-----------------------------------------------------------------------------\n";
print "Results: \n\n";

open IN, $FILE_TMPDATA;

my %total_words, %vals;

while (<IN>){
	if (/$DIR_TEST/){
		my ($filename, $class, %words) = split (/\s+/);
	
		foreach $key(keys %words){
			$total_words{$key}+=$words{$key};
		}
	}
}
close IN;

foreach $key (keys %total_words){
        $vals{$total_words{$key}} .= $key." ";
}

my $max=0, $min=10000;
my %final;

foreach $key (sort {$b <=> $a} keys %vals){
	print "$key\t$vals{$key}\n";
	my @vallist = split(/ /, $vals{$key});
	foreach $val (@vallist){
		$max=$key if ($key>$max);
		$min=$key if ($key<$min);
		$final{$val}=$key;
		# decrease the tag count
		$TAGS--;
		last if !$TAGS;
	}
	last if !$TAGS;
}

my $tags="";
foreach $key (sort keys %final){
	my $class = int(log($final{$key}-$min+1)/log($max-$min)*($SIZENUM-1)+1);
	$tags .= "<a class=\"size$class\">$key</a>\n";
}

open IN, "<$HTML_TEMPLATE";
open OUT, ">$HTML_OUTPUT";

while (<IN>){
	if (/###TAGS###/){
		print OUT $tags;
	}else{
		print OUT;
	}
}
close IN;
close OUT;

print "-----------------------------------------------------------------------------\n";
print "Tag cloud saved in directory $HTML_OUTPUT. Enjoy!\n\n";

if ($FIREFOX_BIN){
	`$FIREFOX_BIN $HTML_OUTPUT`;
}
